#ifndef vorbisDecoder_h
#define vorbisDecoder_h

#include "config.h"

#ifdef HAVE_LIBVORBIS

#include <list>
#include <vector>
#include <vorbis/codec.h>

#include "mediaOutputDecoder.h"
#include "oggPacket.h"
#include "audioPacket.h"
#include "definition.h"
#include "vorbisPosInterpreter.h"
#include "vorbisStreamParameter.h"
#include "streamExtractor.h"

class VorbisDecoder : public MediaOutputDecoder {

 protected:

  vorbis_info      vorbisInfo;
  vorbis_dsp_state vorbisDspState;
  vorbis_block     vorbisBlock;
  vorbis_comment   vorbisComment;

  ExtractorInformation    streamInformation;
  GranulePosInterpreter*  granulePos;

  std::list<OggPacket>    packetList;
  std::vector<OggPacket>  headerList;

  uint8 initCount;
  uint32 packetCount;

  public:

  VorbisDecoder(int8 streamID = 0);
  virtual ~VorbisDecoder();

  virtual std::string getInfoString();
  virtual void init();

  virtual MediaOutputDecoder& operator<<(OggPacket packet);
  virtual MediaOutputDecoder& operator>>(OggPacket& packet);

  VorbisDecoder& operator>>(AudioPacket audioPacket);

  vorbis_comment& getComment();

  void getHeaderList(std::vector<OggPacket>& list);

  virtual uint32 getPositionOfNextPacket();
  virtual int64  getGranulePosOfNextPacket();
  virtual double getTimeOfNextPacket();

};

#endif // WITH_LIBVORBIS

#endif

